@@ -3,10 +3,14 @@
 # vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
 # TextView Object that displays an HTML file
 # Copyright 2005, 2006, 2007 Gustavo J. A. M. Carneiro
-# Changes for Sutekh, Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
+# Changes for Sutekh, Copyright 2008, 2009, 2010
+#           Neil Muller <drnlmuller+sutekh@gmail.com>
 # License: GPL - See COPYRIGHT file for details
 # Original version downloaded from http://www.gnome.org/~gjc/htmltextview.py
 # April 2008
+#   Original version released under the GNU Lesser General Public License
+#   (version 2 or later) - see
+#       http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 
 """A gtk.TextView-based renderer for XHTML-IM, as described in:
    http://www.jabber.org/jeps/jep-0071.html .
@@ -17,39 +21,47 @@ import pango
 import gtk
 import re
 import warnings
-import operator
 import HTMLParser
 from cStringIO import StringIO
 from sutekh.gui.SutekhDialog import SutekhDialog
 from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
 
-oWhiteSpaceRegex = re.compile("\\s+")
+WHITE_SPACE_REGEX = re.compile("\\s+")
+
+# set for fast inclusion queries, since we hit this list multiple times
+HTML_HEADING_TAGS = set(('h1', 'h2', 'h3', 'h4', 'h5', 'h6'))
+
+# don't throw an exception if no screen during import - we'll catch that later
+if gtk.gdk.screen_get_default() is None:
+    SCREEN_RESOLUTION = 0
+else:
+    ## pixels = points * SCREEN_RESOLUTION
+    SCREEN_RESOLUTION = 0.3514598 * (gtk.gdk.screen_height() /
+            float(gtk.gdk.screen_height_mm()))
 
-## pixels = points * fResolution
-fResolution = 0.3514598*(gtk.gdk.screen_height() /
-                    float(gtk.gdk.screen_height_mm()))
 
 def _parse_css_color(sColor):
     '''_parse_css_color(css_color) -> gtk.gdk.Color'''
     if sColor.startswith("rgb(") and sColor.endswith(')'):
-        iRed, iGreen, iBlue = [int(c)*257 for c in sColor[4:-1].split(',')]
+        iRed, iGreen, iBlue = [int(c) * 257 for c in sColor[4:-1].split(',')]
         return gtk.gdk.Color(iRed, iGreen, iBlue)
     else:
         return gtk.gdk.color_parse(sColor)
 
+
 class HtmlHandler(HTMLParser.HTMLParser):
     # pylint: disable-msg=R0201, R0902
     # R0201: can't break these into functions
     # R0902: We need to keep a lot of state to handle HTML properly
-    """Parse the HTML imput and update the gtk.TextView"""
+    """Parse the HTML input and update the gtk.TextView"""
     def __init__(self, oTextView, oStartIter, fLinkLoader):
         HTMLParser.HTMLParser.__init__(self)
         self._oTextBuf = oTextView.get_buffer()
         self._oTextView = oTextView
         self._oIter = oStartIter
         self._sText = ''
-        self._aStyles = [] # a gtk.TextTag or None, for each span level
-        self._aListCounters = [] # stack (top at head) of list
+        self._aStyles = []  # a gtk.TextTag or None, for each span level
+        self._aListCounters = []  # stack (top at head) of list
                                 # counters, or None for unordered list
         self._bInTitle = False
         self._dTargets = {}
@@ -64,77 +76,30 @@ class HtmlHandler(HTMLParser.HTMLParser):
         """Convert background value to TextView background color."""
         oColor = _parse_css_color(sValue)
         oTag.set_property("background-gdk", oColor)
-        if gtk.gtk_version >= (2, 8):
-            oTag.set_property("paragraph-background-gdk", oColor)
+        oTag.set_property("paragraph-background-gdk", oColor)
 
+    def _get_current_attributes(self):
+        """Get current attributes."""
+        aAttrs = self._oTextView.get_default_attributes()
+        self._oIter.backward_char()
+        self._oIter.get_attributes(aAttrs)
+        self._oIter.forward_char()
+        return aAttrs
 
-    if gtk.gtk_version >= (2, 8, 5) or gobject.pygtk_version >= (2, 8, 1):
-
-        def _get_current_attributes(self):
-            """Get current attributes."""
-            aAttrs = self._oTextView.get_default_attributes()
-            self._oIter.backward_char()
-            self._oIter.get_attributes(aAttrs)
-            self._oIter.forward_char()
-            return aAttrs
-
-    else:
-
-        ## Workaround http://bugzilla.gnome.org/show_bug.cgi?id=317455
-        def _get_current_style_attr(self, sPropName, fCombOper=None):
-            """Get the current style attributes."""
-            aTags = [oTag for oTag in self._aStyles if oTag is not None]
-            aTags.reverse()
-            sIsSetName = sPropName + "-set"
-            oValue = None
-            for oTag in aTags:
-                if oTag.get_property(sIsSetName):
-                    if oValue is None:
-                        oValue = oTag.get_property(sPropName)
-                        if fCombOper is None:
-                            return oValue
-                    else:
-                        oValue = fCombOper(oValue,
-                                oTag.get_property(sPropName))
-            return oValue
-
-        class _FakeAttrs(object):
-            """Fake class with default attributes"""
-            __slots__ = ("font", "font_scale")
-            # pylint: disable-msg=C0103
-            # names need to match gtk's convention
-            def __init__(self):
-                self.font_scale = None
-                self.font = None
-
-        def _get_current_attributes(self):
-            """Get current attributes."""
-            # pylint: disable-msg=C0103
-            # font, font-scale required by design
-            aAttrs = self._FakeAttrs()
-            aAttrs.font_scale = self._get_current_style_attr("scale",
-                                                            operator.mul)
-            if aAttrs.font_scale is None:
-                aAttrs.font_scale = 1.0
-            aAttrs.font = self._get_current_style_attr("font-desc")
-            if aAttrs.font is None:
-                aAttrs.font = self._oTextView.style.font_desc
-            return aAttrs
-
-    def _parse_length(self, sValue, bFontRelative, fCallback, *aArgs):
+    def _parse_length(self, sValue, bFontRelative, fCallback, *args):
         """Parse/calc length, converting to pixels.
 
-           calls callback(length, *aArgs) when the length is first computed
+           calls callback(length, *args) when the length is first computed
            or changes
            """
         # pylint: disable-msg=W0142
         # *magic required here
         if sValue.endswith('%'):
-            fFrac = float(sValue[:-1])/100
+            fFrac = float(sValue[:-1]) / 100
             if bFontRelative:
                 oAttrs = self._get_current_attributes()
                 fFontSize = oAttrs.font.get_size() / pango.SCALE
-                fCallback(fFrac * fResolution * fFontSize, *aArgs)
+                fCallback(fFrac * SCREEN_RESOLUTION * fFontSize, *args)
             else:
                 ## CSS says "Percentage values: refer to width of the closest
                 ##           block-level ancestor"
@@ -142,32 +107,34 @@ class HtmlHandler(HTMLParser.HTMLParser):
                 ## textview width instead; a reasonable approximation..
                 oAlloc = self._oTextView.get_allocation()
                 self.__parse_length_frac_cb(self._oTextView, oAlloc,
-                        fFrac, fCallback, aArgs)
+                        fFrac, fCallback, args)
                 self._oTextView.connect("size-allocate",
                         self.__parse_length_frac_cb, fFrac,
-                        fCallback, aArgs)
+                        fCallback, args)
 
-        elif sValue.endswith('pt'): # points
-            fCallback(float(sValue[:-2]) * fResolution, *aArgs)
-        elif sValue.endswith('em'): # ems, the height of the element's font
+        elif sValue.endswith('pt'):  # points
+            fCallback(float(sValue[:-2]) * SCREEN_RESOLUTION, *args)
+        elif sValue.endswith('em'):  # ems, the height of the element's font
             oAttrs = self._get_current_attributes()
             fFontSize = oAttrs.font.get_size() / pango.SCALE
-            fCallback(float(sValue[:-2]) * fResolution * fFontSize, *aArgs)
-        elif sValue.endswith('ex'): # x-height, ~ the height of the letter 'x'
+            fCallback(float(sValue[:-2]) * SCREEN_RESOLUTION * fFontSize,
+                    *args)
+        elif sValue.endswith('ex'):  # x-height, ~ the height of the letter 'x'
             ## FIXME: figure out how to calculate this correctly
             ##        for now 'em' size is used as approximation
             oAttrs = self._get_current_attributes()
             fFontSize = oAttrs.font.get_size() / pango.SCALE
-            fCallback(float(sValue[:-2]) * fResolution * fFontSize, *aArgs)
-        elif sValue.endswith('px'): # pixels
-            fCallback(int(sValue[:-2]), *aArgs)
+            fCallback(float(sValue[:-2]) * SCREEN_RESOLUTION * fFontSize,
+                    *args)
+        elif sValue.endswith('px'):  # pixels
+            fCallback(int(sValue[:-2]), *args)
         else:
             warnings.warn("Unable to parse length value '%s'" % sValue)
 
     @staticmethod
     def __parse_font_size_cb(fLength, oTag):
         """Callback for font size calculations."""
-        oTag.set_property("size-points", fLength/fResolution)
+        oTag.set_property("size-points", fLength / SCREEN_RESOLUTION)
 
     def _parse_style_font_size(self, oTag, sValue):
         """Parse the font size attribute"""
@@ -220,7 +187,7 @@ class HtmlHandler(HTMLParser.HTMLParser):
                 "left-margin")
 
     def _parse_style_margin_right(self, oTag, sValue):
-        """Hanlde the margin right style attribute."""
+        """Handle the margin right style attribute."""
         self._parse_length(sValue, False, self.__frac_length_tag_cb, oTag,
                 "right-margin")
 
@@ -285,12 +252,11 @@ class HtmlHandler(HTMLParser.HTMLParser):
         else:
             warnings.warn("text-decoration:%s not implemented" % sValue)
 
-
     ## build a dictionary mapping styles to methods, for greater speed
     __style_methods = dict()
-    for sStyle in ["background-color", "color", "font-family", "font-size",
+    for sStyle in ("background-color", "color", "font-family", "font-size",
                   "font-style", "font-weight", "margin-left", "margin-right",
-                  "text-align", "text-decoration"]:
+                  "text-align", "text-decoration"):
         try:
             fMethod = locals()["_parse_style_%s" % sStyle.replace('-', '_')]
         except KeyError:
@@ -345,31 +311,31 @@ class HtmlHandler(HTMLParser.HTMLParser):
         self._insert_text(self._sText.replace('\n', ''))
         self._sText = ''
 
-    # pylint: disable-msg=R0913, W0613
+    # pylint: disable-msg=R0913
     # Arguments needed by function signature
-    def _anchor_event(self, oTag, oTextview, oEvent, oIter, oHref, oType_):
+    def _anchor_event(self, _oTag, _oView, oEvent, _oIter, oHref, oType):
         """Something happened to a link, so see if we need to react."""
         if oEvent.type == gtk.gdk.BUTTON_PRESS and oEvent.button == 1:
-            self._oTextView.emit("url-clicked", oHref, oType_)
+            self._oTextView.emit("url-clicked", oHref, oType)
             return True
         return False
 
     # Arguments needed so this can be called via 'size-allocate' event
     def __parse_length_frac_cb(self, oTextView, oAllocation,
-            fFrac, fCallback, aArgs):
+            fFrac, fCallback, args):
         """call the required callback function when the size allocation
            changes."""
         # pylint: disable-msg=W0142
         # *magic required here
-        fCallback(oAllocation.width * fFrac, *aArgs)
+        fCallback(oAllocation.width * fFrac, *args)
 
-    # pylint: enable-msg=R0913, W0613
+    # pylint: enable-msg=R0913
 
     def handle_data(self, sContent):
         """Process the character comments of an element."""
         if self._bInTitle:
             return
-        self._sText += oWhiteSpaceRegex.sub(' ', sContent)
+        self._sText += WHITE_SPACE_REGEX.sub(' ', sContent)
 
     def get_targets(self):
         """Get the list of target anchors."""
@@ -382,11 +348,11 @@ class HtmlHandler(HTMLParser.HTMLParser):
         # This looks like all that textile will produce as entities
         if sEntity == 'lt':
             self.handle_data('<')
-        elif sEntity ==  'gt':
+        elif sEntity == 'gt':
             self.handle_data('>')
-        elif sEntity ==  'amp':
+        elif sEntity == 'amp':
             self.handle_data('&')
-        elif sEntity ==  'quot':
+        elif sEntity == 'quot':
             self.handle_data('"')
         #else:
         #    print 'Unknown entity', sEntity
@@ -423,7 +389,7 @@ class HtmlHandler(HTMLParser.HTMLParser):
                 # Add it to the list of valid targets
                 oMark = self._oTextBuf.create_mark(None, self._oIter, True)
                 self._dTargets[oAttrs['name']] = oMark
-        elif sName in ['h1', 'h2', 'h3', 'h4', 'h5', 'h6']:
+        elif sName in HTML_HEADING_TAGS:
             oTag = self._oTextBuf.create_tag()
             oTag.set_property('underline', pango.UNDERLINE_SINGLE)
             if sName == 'h1':
@@ -459,7 +425,7 @@ class HtmlHandler(HTMLParser.HTMLParser):
             oTextAttrs = self._get_current_attributes()
             fFontSize = oTextAttrs.font.get_size() / pango.SCALE
             iDepth = len(self._aListCounters)
-            oTag.set_property('left-margin', 2.0 * iDepth * fResolution
+            oTag.set_property('left-margin', 2.0 * iDepth * SCREEN_RESOLUTION
                     * fFontSize)
 
         self._begin_span(oStyle, oTag)
@@ -470,7 +436,7 @@ class HtmlHandler(HTMLParser.HTMLParser):
             self._dTargets[oAttrs['id']] = oMark
 
         if sName == 'br':
-            pass # handled in endElement
+            pass  # handled in endElement
         elif sName == 'p':
             if not self._oIter.starts_line():
                 self._insert_text("\n")
@@ -532,7 +498,7 @@ class HtmlHandler(HTMLParser.HTMLParser):
             pass
         elif sName == 'html':
             pass
-        elif sName in ['h1', 'h2', 'h3', 'h4', 'h5', 'h6']:
+        elif sName in HTML_HEADING_TAGS:
             pass
         elif sName == 'em' or sName == 'i':
             pass
@@ -580,7 +546,7 @@ class HtmlHandler(HTMLParser.HTMLParser):
             pass
         elif sName == 'a':
             pass
-        elif sName in ['h1', 'h2', 'h3', 'h4', 'h5', 'h6']:
+        elif sName in HTML_HEADING_TAGS:
             if not self._oIter.starts_line():
                 self._insert_text("\n")
         elif sName == 'em' or sName == 'i':
@@ -623,20 +589,16 @@ class HTMLTextView(gtk.TextView):
         self._dTargets = {}
         self._fLinkLoader = fLinkLoader
 
-    # pylint: disable-msg=W0613
-    # oEvent needed by function signature
-    def __leave_event(self, oWidget, oEvent):
+    def __leave_event(self, oWidget, _oEvent):
         """Cursor has left the widget."""
         if self._bChangedCursor:
             oWindow = oWidget.get_window(gtk.TEXT_WINDOW_TEXT)
             oWindow.set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
             self._bChangedCursor = False
 
-    def __motion_notify_event(self, oWidget, oEvent):
+    def __motion_notify_event(self, oWidget, _oEvent):
         """Change the cursor if the pointer is over a link"""
-        # pylint: disable-msg=W0612
-        # We delibrately ignore oIgnore
-        iXPos, iYPos, oIgnore = oWidget.window.get_pointer()
+        iXPos, iYPos, _oIgnore = oWidget.window.get_pointer()
         iXPos, iYPos = oWidget.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT,
                 iXPos, iYPos)
         aTags = oWidget.get_iter_at_location(iXPos, iYPos).get_tags()
@@ -656,8 +618,6 @@ class HTMLTextView(gtk.TextView):
             self._bChangedCursor = False
         return False
 
-    # pylint: enable-msg=W0613
-
     def set_text_pos(self, sTextAnchor):
         """Set the position to the anchor sTextAnchor"""
         if self._dTargets.has_key(sTextAnchor):
@@ -666,7 +626,7 @@ class HTMLTextView(gtk.TextView):
         # We just silently ignore invalid anchors
 
     def display_html(self, fHTMLInput):
-        """Displa the HTML from the file-like object fHTMLInput"""
+        """Display the HTML from the file-like object fHTMLInput"""
         oBuffer = self.get_buffer()
         oStartOfBuf, oEndOfBuf = oBuffer.get_bounds()
         oBuffer.delete(oStartOfBuf, oEndOfBuf)
@@ -681,8 +641,6 @@ class HTMLTextView(gtk.TextView):
         # Need the anchor -> tag mappings
         self._dTargets = oHandler.get_targets()
 
-if gobject.pygtk_version < (2, 8):
-    gobject.type_register(HTMLTextView)
 
 class HTMLViewDialog(SutekhDialog):
     # pylint: disable-msg=R0904, R0902
@@ -756,37 +714,34 @@ class HTMLViewDialog(SutekhDialog):
     def show_page(self, fInput, sPos=None):
         """Display the html file in fInput in the current window."""
         self._aPastUrls.append((self._fCurrent, self._sTextAnchor))
-        self._aFutureUrls = [] # Forward history is lost
+        self._aFutureUrls = []  # Forward history is lost
         if fInput:
             self._fCurrent = fInput
         self._sTextAnchor = sPos
         self._update_view()
 
-    # pylint: disable-msg=W0613
-    # oWidget, oType required by function signature
-    def _url_clicked(self, oWidget, sUrl, oType):
+    def _url_clicked(self, _oWidget, sUrl, _oType):
         """Update the HTML widget with the new url"""
         aParts = sUrl.split('#')
         sFile = aParts[0]
         if len(aParts) > 1:
-            sPos = aParts[1] # Get anchor
+            sPos = aParts[1]  # Get anchor
         else:
             sPos = None
         if sFile:
-            # pylint: disable-msg=W0703, W0612
+            # pylint: disable-msg=W0703
             # we really do want all the exceptions
-            # oExp is unused
             try:
                 fInput = self._fLinkLoader(sFile)
-            except Exception, oExp:
-                sError = self._sError % { 'missing' : sUrl }
+            except Exception, _oExp:
+                sError = self._sError % {'missing': sUrl}
                 fInput = StringIO(sError)
                 sPos = None
             self.show_page(fInput, sPos)
         else:
             self.show_page(None, sPos)
 
-    def _go_back(self, oWidget):
+    def _go_back(self, _oWidget):
         """Go backwards through the list of visited urls"""
         if len(self._aPastUrls) == 0:
             return
@@ -794,7 +749,7 @@ class HTMLViewDialog(SutekhDialog):
         self._fCurrent, self._sTextAnchor = self._aPastUrls.pop()
         self._update_view()
 
-    def _go_forward(self, oWidget):
+    def _go_forward(self, _oWidget):
         """Go forward through the list of visited urls"""
         if len(self._aFutureUrls) == 0:
             return
