@@ -6,17 +6,20 @@
 
 """Generic multiselect combobox for use in FilterEditor (and elsewhere)"""
 
-import gtk, gobject
+import gtk
 import sys
 from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
+from sutekh.gui.ScrolledList import ScrolledListView
+
 
 def mouse_in_button(oButton):
     """Check if mouse pointer is inside the button"""
-    (iXPos, iYPos) = oButton.get_pointer() # mouse pos relative to button
+    (iXPos, iYPos) = oButton.get_pointer()  # mouse pos relative to button
     oButtonGeom = oButton.allocation
     return (iXPos >= 0) and (iYPos >= 0) and \
             (iXPos < oButtonGeom.width) and (iYPos < oButtonGeom.height)
 
+
 class MultiSelectComboBox(gtk.HBox):
     # pylint: disable-msg=R0904
     # gtk.Widget, so many public methods
@@ -29,12 +32,7 @@ class MultiSelectComboBox(gtk.HBox):
         self._oButton.connect('clicked', self.__show_list)
         self.pack_start(self._oButton)
 
-        self._oListStore = gtk.ListStore(gobject.TYPE_STRING)
-
-        self._oTreeView = gtk.TreeView(self._oListStore)
-        oCell1 = gtk.CellRendererText()
-        oColumn1 = gtk.TreeViewColumn(" ... ", oCell1, markup=0)
-        self._oTreeView.append_column(oColumn1)
+        self._oTreeView = ScrolledListView(" ... ")
         self._oTreeView.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
 
         oScrolled = AutoScrolledWindow(self._oTreeView)
@@ -55,9 +53,7 @@ class MultiSelectComboBox(gtk.HBox):
         self._bInButton = False
         self._oParentWin = oParentWin
 
-    # pylint: disable-msg=W0613
-    # oWidget required by function signature
-    def __grab_event(self, oWidget, oEvent):
+    def __grab_event(self, _oWidget, oEvent):
         """Hook into the event-after chain, so we can check if any uncaught
            events refer to the original button."""
         # This is a bit convuluted, but seems the best
@@ -92,7 +88,7 @@ class MultiSelectComboBox(gtk.HBox):
         # safe, since we're in event-after
         return False
 
-    def __show_list(self, oButton):
+    def __show_list(self, _oButton):
         """Drop down the list of possible selections."""
         self._aOldSelection = self.get_selection()
 
@@ -108,7 +104,7 @@ class MultiSelectComboBox(gtk.HBox):
         tDialogPos = (tWinPos[0] + tButtonPos[0] + tShift[0],
                 tWinPos[1] + tButtonPos[1] + tShift[1])
 
-        self._oDialog.set_keep_above(True) # Keep this above the dialog
+        self._oDialog.set_keep_above(True)  # Keep this above the dialog
         self._oDialog.set_transient_for(self._oParentWin)
         self._oDialog.show_all()
         # WM behaviour means that move is unlikely to work before _oDialog
@@ -116,17 +112,16 @@ class MultiSelectComboBox(gtk.HBox):
         self._oDialog.move(tDialogPos[0], tDialogPos[1])
         self._bInButton = False
 
-    def __hide_on_return(self, oWidget, oEvent):
+    def __hide_on_return(self, _oWidget, oEvent):
         """Hide the list when return or escape is pressed."""
         if oEvent.type is gtk.gdk.KEY_PRESS:
             sKeyName = gtk.gdk.keyval_name(oEvent.keyval)
-            if sKeyName in ['Return', 'Escape']:
+            if sKeyName in ('Return', 'Escape'):
                 if sKeyName == 'Escape':
-                    self.set_selection(self._aOldSelection)
+                    self.set_selected_rows(self._aOldSelection)
                 self.__hide_list()
-                return True # event handled
-        return False # process further
-    # pylint: enable-msg=W0613
+                return True  # event handled
+        return False  # process further
 
     def __hide_list(self):
         """Hide the list of options"""
@@ -143,10 +138,7 @@ class MultiSelectComboBox(gtk.HBox):
 
     def fill_list(self, aVals):
         """Fill the list store with the given values"""
-        self._oListStore.clear()
-        for sEntry in aVals:
-            oIter = self._oListStore.append(None)
-            self._oListStore.set(oIter, 0, sEntry)
+        self._oTreeView.store.fill_list(aVals)
 
     def set_list_size(self, iWidth, iHeight):
         """Set size of the drop-down list"""
@@ -158,24 +150,8 @@ class MultiSelectComboBox(gtk.HBox):
 
     def get_selection(self):
         """Return a list of the selected elements of the list"""
-        aSelectedList = []
-        oModel, oSelection = \
-                self._oTreeView.get_selection().get_selected_rows()
-        for oPath in oSelection:
-            oIter = oModel.get_iter(oPath)
-            sName = oModel.get_value(oIter, 0)
-            aSelectedList.append(sName)
-        return aSelectedList
+        return self._oTreeView.get_selected_data()
 
-    def set_selection(self, aRowsToSelect):
+    def set_selected_rows(self, aRowsToSelect):
         """Set the selected rows in the drop-down to aRowsToSelect"""
-        aRowsToSelect = set(aRowsToSelect)
-        oIter = self._oListStore.get_iter_first()
-        oTreeSelection = self._oTreeView.get_selection()
-        oTreeSelection.unselect_all()
-        while oIter is not None:
-            sName = self._oListStore.get_value(oIter, 0)
-            if sName in aRowsToSelect:
-                oTreeSelection.select_iter(oIter)
-            oIter = self._oListStore.iter_next(oIter)
-        self.__update_button_text()
+        self._oTreeView.set_selected_rows(aRowsToSelect)
