@@ -10,17 +10,18 @@ import time
 from sutekh.core.SutekhObjects import IAbstractCard
 from sutekh.core.ArdbInfo import ArdbInfo
 from sutekh.SutekhInfo import SutekhInfo
-from sutekh.SutekhUtility import pretty_xml
+from sutekh.SutekhUtility import pretty_xml, monger_url, secret_library_url, \
+        norm_xml_quotes
 # pylint: disable-msg=F0401, E0611
 # the allowe failures here makes pylint unhappy
 try:
-    from xml.etree.ElementTree import ElementTree, Element, SubElement
+    from xml.etree.ElementTree import Element, SubElement, tostring
 except ImportError:
-    from elementtree.ElementTree import ElementTree, Element, SubElement
+    from elementtree.ElementTree import Element, SubElement, tostring
 # pylint: enable-msg=F0401, E0611
 
 # Style used by the HTML file
-sHTMLStyle = """
+HTML_STYLE = """
 body {
    background: #000000;
    color: #AAAAAA;
@@ -128,66 +129,19 @@ a:hover {
 }
 """
 
-def _monger_url(oCard, bVamp):
-    """Return a monger url for the given AbstractCard"""
-    sName = oCard.name
-    if sName.startswith('The '):
-        sName = sName[4:] + ', The'
-    elif sName.startswith('An '):
-        sName = sName[3:] + ', An'
-    if bVamp:
-        if oCard.level is not None:
-            sName = sName.replace(' (Advanced)', '')
-            sMongerURL = "http://monger.vekn.org/showvamp.html?NAME=%s ADV" \
-                    % sName
-        else:
-            sMongerURL = "http://monger.vekn.org/showvamp.html?NAME=%s" % sName
-    else:
-        sMongerURL = "http://monger.vekn.org/showcard.html?NAME=%s" % sName
-    # May not need this, but play safe
-    sMongerURL = sMongerURL.replace(' ', '%20')
-    return sMongerURL
-
-def _secret_library_url(oCard, bVamp):
-    """Return a Secret Library url for the given AbstractCard"""
-    sName = oCard.name
-    if sName.startswith('The '):
-        sName = sName[4:] + ',+The'
-    elif sName.startswith('An '):
-        sName = sName[3:] + ',+An'
-    if bVamp:
-        if oCard.level is not None:
-            sName = sName.replace(' (Advanced)', '')
-            sURL = "http://www.secretlibrary.info/?crypt=%s+Adv" \
-                    % sName
-        else:
-            sURL = "http://www.secretlibrary.info/?crypt=%s" \
-                    % sName
-    else:
-        sURL = "http://www.secretlibrary.info/?lib=%s" \
-                    % sName
-    sURL = sURL.replace(' ', '+')
-    return sURL
-
 
 def _sort_vampires(dVamps):
     """Sort the vampires by number, then capacity."""
-    dSortedVampires = {}
-    # pylint: disable-msg=E1101
-    # IAbstrctCard confuses pylint
-    for iId, sVampName, sSet in dVamps:
-        oCard = IAbstractCard(sVampName)
-        iCount = dVamps[(iId, sVampName, sSet)]
+    aSortedVampires = []
+    for oCard, (iCount, _sSet) in dVamps.iteritems():
         if len(oCard.creed) > 0:
             iCapacity = oCard.life
-            sClan = "Imbued"
+            sClan = "%s (Imbued)" % [x.name for x in oCard.creed][0]
         else:
             iCapacity = oCard.capacity
             sClan = [oClan.name for oClan in oCard.clan][0]
-        dSortedVampires.setdefault(oCard, [0, iCapacity, sVampName, sClan])
-        dSortedVampires[oCard][0] += iCount
-    # swap key elements
-    aSortedVampires = [(x[1], x[0]) for x in dSortedVampires.iteritems()]
+        aSortedVampires.append(((iCount, iCapacity, oCard.name, sClan),
+            oCard))
     # We reverse sort by Capacity and Count, normal sort by name
     # fortunately, python's sort is stable, so this works
     aSortedVampires.sort(key=lambda x: x[0][2])
@@ -196,25 +150,17 @@ def _sort_vampires(dVamps):
     # the XSLT approach, but I don't care enough to tweak that
     return aSortedVampires
 
+
 def _sort_lib(dLib):
     """Extract a list of cards sorted into types from the library"""
     dTypes = {}
     # Group by type
-    for tKey in dLib:
-        # pylint: disable-msg=W0612
-        # iId, sSet isn't used here
-        iId, sName, sType, sSet = tKey
-        iCount = dLib[tKey]
-        dTypes.setdefault(sType, {'total' : 0})
-        dTypes[sType]['total'] += iCount
-        dTypes[sType].setdefault(sName, 0)
-        dTypes[sType][sName] += iCount
-    dSortedTypes =  {}
-    for sType, dInfo in dTypes.iteritems():
-        dSortedTypes.setdefault(sType, [dInfo['total']])
-        dSortedTypes[sType].extend([(dInfo[x], x) for x in dInfo
-                if x != 'total'])
-    return sorted(dSortedTypes.items())
+    for oCard, (iCount, sType, _sSet) in dLib.iteritems():
+        dTypes.setdefault(sType, [0])
+        dTypes[sType][0] += iCount
+        dTypes[sType].append((iCount, oCard.name))
+    return sorted(dTypes.items())
+
 
 def _add_span(oElement, sText, sClass=None, sId=None):
     """Add a span element to the element oElement"""
@@ -225,6 +171,7 @@ def _add_span(oElement, sText, sClass=None, sId=None):
     if sId:
         oSpan.attrib['id'] = sId
 
+
 def _add_text(oElement, oCard):
     """Add the card text to the ElementTree line by line"""
     oTextDiv = SubElement(oElement, "div")
@@ -233,6 +180,7 @@ def _add_text(oElement, oCard):
         oPara = SubElement(oTextDiv, 'p')
         oPara.text = sLine
 
+
 class WriteArdbHTML(ArdbInfo):
     """Export a Card set to a 'nice' HTML file.
 
@@ -241,38 +189,36 @@ class WriteArdbHTML(ArdbInfo):
        This tries to match the HTML file produced by ARDB.
        """
 
-    def __init__(self, sLinkMode='Monger'):
+    def __init__(self, sLinkMode='Monger', bDoText=False):
         super(WriteArdbHTML, self).__init__()
         self._sLinkMode = sLinkMode
+        self._bDoText = bDoText
 
-    def write(self, sFileName, oCardSet, oCardIter, bDoText=False):
+    def write(self, fOut, oHolder):
         """Handle the response to the dialog"""
         # pylint: disable-msg=E1101
         # SQLObject methods confuse pylint
-        # 'w' should be fine, since we're using an ascii coding
-        fOut = file(sFileName, "w")
-        oETree = self.gen_tree(oCardSet, oCardIter, bDoText)
+        oRoot = self._gen_tree(oHolder)
         # We're producing XHTML output, so we need a doctype header
         fOut.write('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0'
                 ' Strict//EN"\n "http://www.w3.org/TR/xhtml1/DTD/'
                 'xhtml1-strict.dtd">\n')
-        # We have the elementree with the needed information,
-        # need to produce decent HTML output
-        oETree.write(fOut)
-        fOut.close()
+        sData = tostring(oRoot)
+        sData = norm_xml_quotes(sData)
+        fOut.write(sData)
 
-    def gen_tree(self, oCardSet, oCardIter, bDoText):
+    def _gen_tree(self, oHolder):
         """Convert the Cards to a element tree containing 'nice' HTML"""
         oDocRoot = Element('html', xmlns='http://www.w3.org/1999/xhtml',
                 lang='en')
         oDocRoot.attrib["xml:lang"] = 'en'
 
-        oBody = self._add_header(oDocRoot, oCardSet)
+        oBody = self._add_header(oDocRoot, oHolder)
 
-        dCards = self._get_cards(oCardIter)
+        dCards = self._get_cards(oHolder.cards)
         aSortedVampires = self._add_crypt(oBody, dCards)
         aSortedLibCards = self._add_library(oBody, dCards)
-        if bDoText:
+        if self._bDoText:
             oCardText = SubElement(oBody, "div", id="cardtext")
             oTextHead = SubElement(oCardText, "h3")
             oTextHead.attrib["class"] = "cardtext"
@@ -288,12 +234,12 @@ class WriteArdbHTML(ArdbInfo):
                 "generator")
 
         pretty_xml(oDocRoot)
-        return ElementTree(oDocRoot)
+        return oDocRoot
 
     # methods to fill in the actual HTML content
     # pylint: disable-msg=R0201
     # these are all methods for consistency
-    def _add_header(self, oDocRoot, oCardSet):
+    def _add_header(self, oDocRoot, oHolder):
         """Add the header and title of the HTML file."""
         oHead = SubElement(oDocRoot, 'head')
         oEncoding = SubElement(oHead, 'meta')
@@ -307,23 +253,26 @@ class WriteArdbHTML(ArdbInfo):
         oEncoding.attrib['content'] = 'text/html; charset="us-ascii"'
         oStyle = SubElement(oHead, 'style', type="text/css")
         # Is there a better idea here?
-        oStyle.text = sHTMLStyle
+        oStyle.text = HTML_STYLE
         oTitle = SubElement(oHead, "title")
-        oTitle.text = "VTES deck : %s by %s" % (oCardSet.name,
-                oCardSet.author)
+        if oHolder.author:
+            oTitle.text = "VTES deck : %s by %s" % (oHolder.name,
+                oHolder.author)
+        else:
+            oTitle.text = "VTES deck : %s" % oHolder.name
 
         oBody = SubElement(oDocRoot, "body")
         oInfo = SubElement(oBody, "div", id="info")
         oName = SubElement(oInfo, "h1", id="nametitle")
         _add_span(oName, 'Deck Name :')
-        _add_span(oName, oCardSet.name, 'value', 'namevalue')
+        _add_span(oName, oHolder.name, 'value', 'namevalue')
         oAuthor = SubElement(oInfo, "h2", id="authortitle")
         _add_span(oAuthor, 'Author : ')
-        _add_span(oAuthor, oCardSet.author, 'value', 'authornamevalue')
+        _add_span(oAuthor, oHolder.author, 'value', 'authornamevalue')
         oDesc = SubElement(oInfo, "h2", id="description")
         _add_span(oDesc, 'Description : ')
         oPara = SubElement(oInfo, "p")
-        _add_span(oPara, oCardSet.comment, 'value', 'descriptionvalue')
+        _add_span(oPara, oHolder.comment, 'value', 'descriptionvalue')
         return oBody
 
     def _gen_link(self, oCard, oSpan, sName, bVamp):
@@ -332,10 +281,10 @@ class WriteArdbHTML(ArdbInfo):
         oRowHREF = oSpan
         if self._sLinkMode == 'Monger':
             oRowHREF = SubElement(oSpan, "a",
-                    href=_monger_url(oCard, bVamp))
+                    href=monger_url(oCard, bVamp))
         elif self._sLinkMode == 'Secret Library':
             oRowHREF = SubElement(oSpan, "a",
-                    href=_secret_library_url(oCard, bVamp))
+                    href=secret_library_url(oCard, bVamp))
         if bVamp:
             oRowHREF.text = sName.replace(' (Advanced)', '')
         else:
@@ -353,7 +302,7 @@ class WriteArdbHTML(ArdbInfo):
             _add_span(oCryptTitle, 'Crypt')
             _add_span(oCryptTitle, "[%(size)d vampires] Capacity min : %(min)d"
                     " max : %(max)d average : %(avg).2f" % dCryptStats)
-            aSortedVampires = _sort_vampires(dVamps)
+            aSortedVampires = _sort_vampires(self._group_sets(dVamps))
             return oCrypt, aSortedVampires
 
         def add_row(oCryptTBody, tVampInfo, oCard):
@@ -404,7 +353,7 @@ class WriteArdbHTML(ArdbInfo):
         def start_section(oBody, dCards):
             """Set up the header for this section"""
             (dLib, iLibSize) = self._extract_library(dCards)
-            aSortedLibCards = _sort_lib(dLib)
+            aSortedLibCards = _sort_lib(self._group_sets(dLib))
             oLib = SubElement(oBody, "div", id="library")
             oLibTitle = SubElement(oLib, "h3", id="librarytitle")
             _add_span(oLibTitle, "Library")
