@@ -9,6 +9,7 @@
 
 import codecs
 import urllib2
+import logging
 
 WW_CARDLIST_URL = "http://www.vekn.net/images/stories/downloads/cardlist.txt"
 
@@ -60,6 +61,25 @@ WW_RULINGS_URL = [
         ]
 
 
+def guess_encoding(sData, sFile):
+    """Try to determine the correct encoding from the data"""
+    # Try utf8 first, then iso8859-1, which covers what we've
+    # seen so far. We may also need one of the cp codecs, but
+    # it currently seems not.
+    aEncodings = ['utf8', 'iso8859-1']
+    for sEnc in aEncodings:
+        try:
+            logging.info('Trying %s for %s', sEnc, sFile)
+            sData.decode(sEnc)
+            return sEnc
+        except UnicodeDecodeError:
+            # We move onto the next one
+            pass
+    # We've fallen off the back of the loop, so we're in trouble
+    # and there're no good choices left, so we raise a RuntimeError
+    raise RuntimeError('Unable to indentify correct encoding for %s' % sFile)
+
+
 class WwFile(object):
     """WwFile is a convenience class which has an .open(..) method which
        returns a file-like object with the encoding set correctly.
@@ -85,6 +105,16 @@ class WwFile(object):
         if self.bFileObj:
             return self.sfFile
         elif self.bUrl:
-            return codecs.EncodedFile(urllib2.urlopen(self.sfFile), 'utf8')
+            oFile = urllib2.urlopen(self.sfFile)
+            sData = oFile.read(1000)
+            sFileEnc = guess_encoding(sData, self.sfFile)
+            oFile.close()
+            return codecs.EncodedFile(urllib2.urlopen(self.sfFile), 'utf8',
+                    sFileEnc)
         else:
-            return codecs.EncodedFile(open(self.sfFile, 'rU'), 'utf8')
+            oFile = open(self.sfFile, 'rU')
+            sData = oFile.read(1000)
+            sFileEnc = guess_encoding(sData, self.sfFile)
+            oFile.close()
+            return codecs.EncodedFile(open(self.sfFile, 'rU'), 'utf8',
+                    sFileEnc)
